package space.kklochko.spring_rest_example.config;

import java.util.List;

import jakarta.persistence.EntityManager;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import space.kklochko.spring_rest_example.db.factories.EntityManagerConnection;
import space.kklochko.spring_rest_example.db.repositories.IndicatorRepository;
import space.kklochko.spring_rest_example.interceptors.AuthorizedInterceptor;
import space.kklochko.spring_rest_example.models.factories.TokenFactory;
import space.kklochko.spring_rest_example.security.tokens.SimpleRandomToken;
import space.kklochko.spring_rest_example.security.tokens.StringTokenFactory;

@EnableWebMvc
@Configuration
@ComponentScan("space.kklochko")
public class WebConfig implements WebMvcConfigurer {
    @Override
    public void configureMessageConverters(List<HttpMessageConverter<?>> converters) {
        converters.add(new MappingJackson2HttpMessageConverter());
    }

    @Bean
    public EntityManager manager() {
        return (new EntityManagerConnection("spring_rest_example")).connect();
    }

    @Bean
    public IndicatorRepository indicatorRepository(EntityManager manager) {
        return new IndicatorRepository(manager);
    }

    @Bean
    public TokenFactory tokenFactory() {
        StringTokenFactory stringTokenFactory = new SimpleRandomToken();
        return new TokenFactory(stringTokenFactory);
    }

    @Bean
    public AuthorizedInterceptor authorizedInterceptor() {
        return new AuthorizedInterceptor();
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(authorizedInterceptor())
                .addPathPatterns("/api/v1/departments/**")
                .addPathPatterns("/api/v1/patients/**")
                .addPathPatterns("/api/v1/indicators/**");
    }
}
