package space.kklochko.spring_rest_example.db.repositories;

import jakarta.persistence.EntityGraph;
import jakarta.persistence.EntityManager;
import jakarta.persistence.TypedQuery;
import org.springframework.stereotype.Component;
import space.kklochko.spring_rest_example.db.repositories.blocks.TransactionalInsert;
import space.kklochko.spring_rest_example.db.repositories.blocks.TransactionalRemove;
import space.kklochko.spring_rest_example.db.repositories.blocks.TransactionalUpdate;
import space.kklochko.spring_rest_example.models.Department;
import space.kklochko.spring_rest_example.models.Patient;

import java.util.List;
import java.util.UUID;

@Component
public class DepartmentRepository extends AbstractRepository<Department> {
    public DepartmentRepository(EntityManager manager){
        super(manager);
    }

    public boolean create(Department object) {
        TransactionalInsert insert = new TransactionalInsert();

        return insert.run(manager, object);
    }

    public Department read(UUID id) {
        return manager.find(Department.class, id);
    }

    public List<Department> readAll() {
        EntityGraph<Department> entityGraph = manager.createEntityGraph(Department.class);
        entityGraph.addAttributeNodes("patients");
        //entityGraph.addSubgraph("patients")
                   //.addAttributeNodes("indicators");

        TypedQuery<Department> query = manager.createQuery("SELECT d FROM Department d", Department.class);
        query.setHint( "javax.persistence.fetchgraph", entityGraph);

        List<Department> departments = query.getResultList();

        for(Department department : departments) {
            EntityGraph<Patient> patientEntityGraph = manager.createEntityGraph(Patient.class);
            patientEntityGraph.addAttributeNodes("indicators");

            TypedQuery<Patient> patientQuery = manager.createQuery("SELECT p FROM Patient p", Patient.class);
            patientQuery.setHint( "javax.persistence.fetchgraph", patientEntityGraph);

            department.setPatients(patientQuery.getResultList());
        }

        return departments;
    }

    public boolean update(Department object) {
        TransactionalUpdate update = new TransactionalUpdate();

        return update.run(manager, object);
    }

    public boolean delete(UUID id) {
        Department object = read(id);

        TransactionalRemove remove = new TransactionalRemove();

        return remove.run(manager, object);
    }
}
